/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.heuristic.api;

import ru.ulstu.extractor.heuristic.component.BuildTool;
import ru.ulstu.extractor.heuristic.component.ProgrammingLanguage;
import ru.ulstu.extractor.heuristic.model.StructuralUnit;
import ru.ulstu.extractor.heuristic.service.DetectorService;
import ru.ulstu.extractor.util.StringUtils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

public abstract class StructuralUnitIdentifier {
    public List<StructuralUnit> getEntityClasses(String projectPath, List<File> projectFiles, List<File> rootProjectFiles) {
        String subDirectory = getSourceDirectory(rootProjectFiles);
        return getEntityClasses(projectPath, subDirectory, projectFiles);
    }

    protected abstract String getSourceDirectory(List<File> rootProjectFiles);

    public abstract List<StructuralUnit> getEntityClasses(String projectPath, String subDirectory, List<File> projectFiles);

    public abstract List<StructuralUnit> getBusinessLogicClasses();

    public abstract boolean isMultiModuleProject();

    public abstract Optional<BuildTool> getBuildTool(List<File> rootDirectoryFiles);

    protected abstract DetectorService getDetectorService();

    protected abstract ProgrammingLanguage getProgrammingLanguage();

    protected abstract boolean isEntityClass(File file) throws FileNotFoundException;

    protected abstract boolean isBusinessLogicClass(File file);

    protected Optional<ProgrammingLanguage> getMainProgrammingLanguage(String projectPath, List<File> projectFiles, List<File> rootProjectFiles) {
        String subDirectory = getSourceDirectory(rootProjectFiles);
        Map<String, Integer> projectFileLanguageFrequency = new HashMap<>();
        projectFiles.stream()
                .filter(file -> StringUtils.fileInSubdirectory(file.getPath(), projectPath, subDirectory))
                .forEach(projectFile -> {
                    try {
                        String detectedLanguage = getDetectorService().getDetectedLanguage(
                                projectFile.getName(),
                                new String(Files.readAllBytes(projectFile.toPath())),
                                DetectorService.LangDetectScrupulousness.LOW);
                        projectFileLanguageFrequency.put(detectedLanguage, projectFileLanguageFrequency.getOrDefault(detectedLanguage, 0) + 1);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                });

        Optional<Map.Entry<String, Integer>> mostFrequentLanguageEntry = projectFileLanguageFrequency
                .entrySet()
                .stream()
                .max(Map.Entry.comparingByValue());
        if (mostFrequentLanguageEntry.isEmpty()) {
            return Optional.empty();
        }

        ProgrammingLanguage programmingLanguage = getProgrammingLanguage();
        return programmingLanguage.getName().equals(mostFrequentLanguageEntry.get().getKey())
                ? Optional.of(programmingLanguage)
                : Optional.empty();
    }

    public abstract boolean canAppliedToProject(String projectPath, List<File> projectFiles, List<File> rootProjectFiles);
}
